@@ -30,6 +30,134 @@
 
 namespace grn {
   namespace {
+    struct Int8Handler : public rapidjson::BaseReaderHandler<>
+    {
+      grn_ctx *ctx_;
+      grn_obj *uvector_;
+
+      Int8Handler(grn_ctx *ctx,
+                  grn_obj *uvector,
+                  bool add_record_if_not_exist)
+        : ctx_(ctx),
+          uvector_(uvector) {
+      }
+
+      bool Default() {return false;}
+
+      bool Int(int value) {
+        GRN_INT8_PUT(ctx_, uvector_, value);
+        return true;
+      }
+
+      bool Uint(unsigned int value) {
+        return Int(value);
+      }
+
+      bool Int64(int64_t value) {
+        return Int(value);
+      }
+
+      bool Uint64(uint64_t value) {
+        return Int(value);
+      }
+    };
+
+    struct UInt8Handler : public rapidjson::BaseReaderHandler<>
+    {
+      grn_ctx *ctx_;
+      grn_obj *uvector_;
+
+      UInt8Handler(grn_ctx *ctx,
+                   grn_obj *uvector,
+                   bool add_record_if_not_exist)
+        : ctx_(ctx),
+          uvector_(uvector) {
+      }
+
+      bool Default() {return false;}
+
+      bool Int(int value) {
+        return Uint(value);
+      }
+
+      bool Uint(unsigned int value) {
+        GRN_UINT8_PUT(ctx_, uvector_, value);
+        return true;
+      }
+
+      bool Int64(int64_t value) {
+        return Uint(value);
+      }
+
+      bool Uint64(uint64_t value) {
+        return Uint(value);
+      }
+    };
+
+    struct Int16Handler : public rapidjson::BaseReaderHandler<>
+    {
+      grn_ctx *ctx_;
+      grn_obj *uvector_;
+
+      Int16Handler(grn_ctx *ctx,
+                   grn_obj *uvector,
+                   bool add_record_if_not_exist)
+        : ctx_(ctx),
+          uvector_(uvector) {
+      }
+
+      bool Default() {return false;}
+
+      bool Int(int value) {
+        GRN_INT16_PUT(ctx_, uvector_, value);
+        return true;
+      }
+
+      bool Uint(unsigned int value) {
+        return Int(value);
+      }
+
+      bool Int64(int64_t value) {
+        return Int(value);
+      }
+
+      bool Uint64(uint64_t value) {
+        return Int(value);
+      }
+    };
+
+    struct UInt16Handler : public rapidjson::BaseReaderHandler<>
+    {
+      grn_ctx *ctx_;
+      grn_obj *uvector_;
+
+      UInt16Handler(grn_ctx *ctx,
+                    grn_obj *uvector,
+                    bool add_record_if_not_exist)
+        : ctx_(ctx),
+          uvector_(uvector) {
+      }
+
+      bool Default() {return false;}
+
+      bool Int(int value) {
+        return Uint(value);
+      }
+
+      bool Uint(unsigned int value) {
+        GRN_UINT16_PUT(ctx_, uvector_, value);
+        return true;
+      }
+
+      bool Int64(int64_t value) {
+        return Uint(value);
+      }
+
+      bool Uint64(uint64_t value) {
+        return Uint(value);
+      }
+    };
+
     struct Int32Handler : public rapidjson::BaseReaderHandler<>
     {
       grn_ctx *ctx_;
@@ -62,6 +190,102 @@ namespace grn {
       }
     };
 
+    struct UInt32Handler : public rapidjson::BaseReaderHandler<>
+    {
+      grn_ctx *ctx_;
+      grn_obj *uvector_;
+
+      UInt32Handler(grn_ctx *ctx,
+                    grn_obj *uvector,
+                    bool add_record_if_not_exist)
+        : ctx_(ctx),
+          uvector_(uvector) {
+      }
+
+      bool Default() {return false;}
+
+      bool Int(int value) {
+        return Uint(value);
+      }
+
+      bool Uint(unsigned int value) {
+        GRN_UINT32_PUT(ctx_, uvector_, value);
+        return true;
+      }
+
+      bool Int64(int64_t value) {
+        return Uint(value);
+      }
+
+      bool Uint64(uint64_t value) {
+        return Uint(value);
+      }
+    };
+
+    struct Int64Handler : public rapidjson::BaseReaderHandler<>
+    {
+      grn_ctx *ctx_;
+      grn_obj *uvector_;
+
+      Int64Handler(grn_ctx *ctx,
+                   grn_obj *uvector,
+                   bool add_record_if_not_exist)
+        : ctx_(ctx),
+          uvector_(uvector) {
+      }
+
+      bool Default() {return false;}
+
+      bool Int(int value) {
+        return Int64(value);
+      }
+
+      bool Uint(unsigned int value) {
+        return Int64(value);
+      }
+
+      bool Int64(int64_t value) {
+        GRN_INT64_PUT(ctx_, uvector_, value);
+        return true;
+      }
+
+      bool Uint64(uint64_t value) {
+        return Int64(value);
+      }
+    };
+
+    struct UInt64Handler : public rapidjson::BaseReaderHandler<>
+    {
+      grn_ctx *ctx_;
+      grn_obj *uvector_;
+
+      UInt64Handler(grn_ctx *ctx,
+                    grn_obj *uvector,
+                    bool add_record_if_not_exist)
+        : ctx_(ctx),
+          uvector_(uvector) {
+      }
+
+      bool Default() {return false;}
+
+      bool Int(int value) {
+        return Uint64(value);
+      }
+
+      bool Uint(unsigned int value) {
+        return Uint64(value);
+      }
+
+      bool Int64(int64_t value) {
+        return Uint64(value);
+      }
+
+      bool Uint64(uint64_t value) {
+        GRN_UINT64_PUT(ctx_, uvector_, value);
+        return true;
+      }
+    };
+
     struct TableWeightHandler : public rapidjson::BaseReaderHandler<>
     {
       TableWeightHandler(grn_ctx *ctx,
@@ -676,6 +900,29 @@ namespace grn {
       std::vector<ContainerType> container_type_stack_;
     };
 
+    struct TextHandler : public rapidjson::BaseReaderHandler<>
+    {
+      grn_ctx *ctx_;
+      grn_obj *vector_;
+
+      TextHandler(grn_ctx *ctx, grn_obj *vector)
+        : ctx_(ctx),
+          vector_(vector) {
+      }
+
+      bool Default() {return false;}
+
+      bool String(const char *data, size_t size, bool copy) {
+        grn_vector_add_element_float(ctx_,
+                                     vector_,
+                                     data,
+                                     size,
+                                     0.0,
+                                     GRN_DB_SHORT_TEXT);
+        return true;
+      }
+    };
+
     template <typename Handler>
     grn_rc
     json_to_uvector(grn_ctx *ctx,
@@ -714,6 +961,25 @@ namespace grn {
         return GRN_INVALID_ARGUMENT;
       }
     }
+
+    grn_rc
+    json_to_text_vector(grn_ctx *ctx,
+                        rapidjson::Document *document,
+                        grn_obj *dest) {
+      TextHandler handler(ctx, dest);
+      if (document->IsArray()) {
+        auto n = document->Size();
+        for (size_t i = 0; i < n; ++i) {
+          const auto &element = (*document)[i];
+          if (!element.Accept(handler)) {
+            return GRN_INVALID_ARGUMENT;
+          }
+        }
+        return GRN_SUCCESS;
+      } else {
+        return GRN_INVALID_ARGUMENT;
+      }
+    }
   }
 
   grn_rc
@@ -742,11 +1008,46 @@ namespace grn {
       return rc;
     }
     switch (dest->header.domain) {
+    case GRN_DB_INT8 :
+      return json_to_uvector<Int8Handler>(ctx,
+                                          &document,
+                                          dest,
+                                          add_record_if_not_exist);
+    case GRN_DB_UINT8 :
+      return json_to_uvector<UInt8Handler>(ctx,
+                                           &document,
+                                           dest,
+                                           add_record_if_not_exist);
+    case GRN_DB_INT16 :
+      return json_to_uvector<Int16Handler>(ctx,
+                                           &document,
+                                           dest,
+                                           add_record_if_not_exist);
+    case GRN_DB_UINT16 :
+      return json_to_uvector<UInt16Handler>(ctx,
+                                            &document,
+                                            dest,
+                                            add_record_if_not_exist);
     case GRN_DB_INT32 :
       return json_to_uvector<Int32Handler>(ctx,
                                            &document,
                                            dest,
                                            add_record_if_not_exist);
+    case GRN_DB_UINT32 :
+      return json_to_uvector<UInt32Handler>(ctx,
+                                            &document,
+                                            dest,
+                                            add_record_if_not_exist);
+    case GRN_DB_INT64 :
+      return json_to_uvector<Int64Handler>(ctx,
+                                           &document,
+                                           dest,
+                                           add_record_if_not_exist);
+    case GRN_DB_UINT64 :
+      return json_to_uvector<UInt64Handler>(ctx,
+                                            &document,
+                                            dest,
+                                            add_record_if_not_exist);
     default :
       {
         grn_rc rc = GRN_INVALID_ARGUMENT;
@@ -769,6 +1070,20 @@ namespace grn {
       }
     }
   }
+
+  grn_rc
+  cast_text_to_text_vector(grn_ctx *ctx,
+                           grn_obj *src,
+                           grn_obj *dest)
+  {
+    rapidjson::Document document;
+    rapidjson::MemoryStream stream(GRN_TEXT_VALUE(src), GRN_TEXT_LEN(src));
+    document.ParseStream(stream);
+    if (document.HasParseError()) {
+      return GRN_INVALID_ARGUMENT;
+    }
+    return json_to_text_vector(ctx, &document, dest);
+  }
 }
 #endif
 
@@ -783,6 +1098,18 @@ extern "C" {
     return grn::cast_text_to_uvector(ctx, src, dest, add_record_if_not_exist);
 #else
     return GRN_INVALID_ARGUMENT;
+#endif
+  }
+
+  grn_rc
+  grn_obj_cast_text_to_text_vector(grn_ctx *ctx,
+                                   grn_obj *src,
+                                   grn_obj *dest)
+  {
+#ifdef GRN_WITH_RAPIDJSON
+    return grn::cast_text_to_text_vector(ctx, src, dest);
+#else
+    return GRN_INVALID_ARGUMENT;
 #endif
   }
 }
